# Generated by roxygen2: do not edit by hand

S3method(chat_history,character)
S3method(chat_history,data.frame)
S3method(chat_history,default)
S3method(print,Tidyprompt)
S3method(prompt_wrap,default)
S3method(prompt_wrap,tidyprompt)
export("llm_provider-class")
export("persistent_chat-class")
export("tidyprompt-class")
export(add_image)
export(add_msg_to_chat_history)
export(add_text)
export(answer_as_boolean)
export(answer_as_category)
export(answer_as_dataframe)
export(answer_as_integer)
export(answer_as_json)
export(answer_as_key_value)
export(answer_as_list)
export(answer_as_multi_category)
export(answer_as_named_list)
export(answer_as_numeric)
export(answer_as_regex_match)
export(answer_as_text)
export(answer_by_chain_of_thought)
export(answer_by_react)
export(answer_using_r)
export(answer_using_sql)
export(answer_using_tools)
export(chat_history)
export(construct_prompt_text)
export(df_to_string)
export(extract_from_return_list)
export(get_chat_history)
export(get_prompt_wraps)
export(is_tidyprompt)
export(llm_break)
export(llm_break_soft)
export(llm_feedback)
export(llm_provider_ellmer)
export(llm_provider_google_gemini)
export(llm_provider_groq)
export(llm_provider_mistral)
export(llm_provider_ollama)
export(llm_provider_openai)
export(llm_provider_openrouter)
export(llm_provider_xai)
export(llm_verify)
export(prompt_wrap)
export(provider_prompt_wrap)
export(quit_if)
export(r_json_schema_to_example)
export(send_prompt)
export(set_chat_history)
export(set_system_prompt)
export(skim_with_labels_and_levels)
export(tidyprompt)
export(tools_add_docs)
export(tools_get_docs)
export(user_verify)
export(vector_list_to_string)
importFrom(R6,R6Class)
importFrom(rlang,"%||%")
importFrom(rlang,set_names)
